@@ -33,7 +33,6 @@
 # 2FIX:
 # add britten-jones weights CI
 
-
 ########################################################################
 # Tests 
 ########################################################################
@@ -42,7 +41,7 @@
 
 # the guts of a SR equality test, hinging on different
 # covariance functions.
-.sr_eq_guts <- function(X,contrasts,estfunc=cov) {
+.sr_eq_guts <- function(X,contrasts,vcov.func=vcov) {
 	X <- na.omit(X)
 	n <- dim(X)[1]
 	p <- dim(X)[2]
@@ -52,32 +51,16 @@
 	if (dim(contrasts)[2] != p)
 		stop("size mismatch in 'X', 'contrasts'")
 
-	# cat together X and X squared
-	XX2 <- cbind(X,X^2)
-	mm2 <- colMeans(XX2)
+	retval <- sr_vcov(X,vcov.func=vcov.func)
 
-	m1 <- mm2[1:p]
-	m2 <- mm2[p + (1:p)]
-	# the SR:
-	SR <- m1 / sqrt(m2 - m1^2)
-
-	# construct/estimate Sigma hat
-	Shat = estfunc(XX2)
-
-	# construct D matrix
-	deno <- (m2 - m1^2)^(3/2)
-	D1 <- diag(m2 / deno)
-	D2 <- diag(-m1 / (2*deno))
-	Dt <- rbind(D1,D2)
-
-	# Omegahat
-	Ohat <- t(Dt) %*% Shat %*% Dt
+	# store these:
+	retval$n <- n
+	retval$k <- k
 
 	# the test statistic:
-	ESR <- contrasts %*% SR
-	COC <- contrasts %*% Ohat %*% t(contrasts)
+	retval$ESR <- contrasts %*% retval$SR
+	retval$COC <- contrasts %*% retval$Ohat %*% t(contrasts)
 
-	retval <- list(n=n,k=k,p=p,SR=SR,ESR=ESR,COC=COC)
 	return(retval)
 }
 #' @title Paired test for equality of Sharpe ratio
@@ -111,7 +94,8 @@
 #'
 #' sr_equality_test(X,type=c("chisq","F","t"),
 #'                  alternative=c("two.sided","less","greater"),
-#'                  contrasts=NULL)
+#'                  contrasts=NULL,
+#'                  vcov.func=vcov) 
 #'
 #' @param X an \eqn{n \times p}{n x p} matrix of paired observations.
 #' @param contrasts an \eqn{k \times p}{k x p} matrix of the contrasts
@@ -124,6 +108,12 @@
 #'        \code{"less"}. You can specify just the initial letter.
 #'        This is only relevant for the \code{"t"} test.
 #'        \code{"greater"} corresponds to \eqn{H_a: E s > 0}{Ha: E s > 0}.
+#' @param vcov.func a function which takes a model of class lm (one of
+#'        the form x ~ 1), and produces a variance-covariance matrix.
+#'        The default is \code{\link{vcov}}, which produces a 'vanilla'
+#'        estimate of covariance. Other sensible options are
+#'        \code{vcovHAC} from the \code{sandwich} package.
+#'
 #' @keywords htest
 #' @return Object of class \code{htest}, a list of the test statistic,
 #' the size of \code{X}, and the \code{method} noted.
@@ -145,6 +135,9 @@
 #' Memmel, C. "Performance hypothesis testing with the Sharpe ratio." Finance
 #' Letters 1 (2003): 21--23.
 #'
+#' @template ref-LW
+#' @template ref-Lo
+#'
 #' @examples 
 #' # under the null 
 #' rv <- sr_equality_test(matrix(rnorm(500*5),ncol=5))
@@ -174,35 +167,60 @@
 #' plot(ecdf(pvs))
 #' abline(0,1,col='red') 
 #' }
+#' \dontrun{
+#' if (require(sandwich)) {
+#'   set.seed(as.integer(charToRaw("0b2fd4e9-3bdf-4e3e-9c75-25c6d18c331f")))
+#'   n.manifest <- 10
+#'   n.latent <- 4
+#'   n.day <- 1024
+#'   snr <- 0.95
+#'   latent.rets <- matrix(rnorm(n.day*n.latent),ncol=n.latent) %*%
+#'		matrix(runif(n.latent*n.manifest),ncol=n.manifest)
+#'   noise.rets <- matrix(rnorm(n.day*n.manifest),ncol=n.manifest)
+#'   some.rets <- snr * latent.rets + sqrt(1-snr^2) * noise.rets
+#'   # naive vcov
+#'   pvs0 <- sr_equality_test(some.rets)
+#'   # HAC vcov
+#'   pvs1 <- sr_equality_test(some.rets,vcov.func=vcovHAC)
+#'   # more elaborately:
+#'   pvs <- sr_equality_test(some.rets,vcov.func=function(amod) {
+#'		vcovHAC(amod,prewhite=TRUE) })
+#' }
+#' }
 #'
 #'@export
 sr_equality_test <- function(X,type=c("chisq","F","t"),
 														 alternative=c("two.sided","less","greater"),
-														 contrasts=NULL) {
+														 contrasts=NULL,
+														 vcov.func=vcov) {
 	# all this stolen from t.test.default:
 	alternative <- match.arg(alternative)
 	dname <- deparse(substitute(X))
-	subprob <- .sr_eq_guts(X,contrasts,estfunc=cov)
-	type <- match.arg(type)
 
-	if ((type == "t") && (subprob$k != 1))
+	# delegate
+	srmom <- .sr_eq_guts(X,contrasts,vcov.func=vcov.func)
+
+	type <- match.arg(type)
+	if ((type == "t") && (srmom$k != 1))
 		stop("can only perform t-test on single contrast");
 
 	if (type == "t") {
-		ts <- subprob$ESR * sqrt(subprob$n / subprob$COC)
+		#ts <- srmom$ESR * sqrt(srmom$n / srmom$COC)
+		ts <- srmom$ESR * sqrt(1 / srmom$COC)
 		names(ts) <- "t"
 		pval <- switch(alternative,
-									 two.sided = .oneside2two(pt(ts,df=subprob$n-1)),
-									 less = pt(ts,df=subprob$n-1,lower.tail=TRUE),
-									 greater = pt(ts,df=subprob$n-1,lower.tail=FALSE))
+									 two.sided = .oneside2two(pt(ts,df=srmom$n-1)),
+									 less = pt(ts,df=srmom$n-1,lower.tail=TRUE),
+									 greater = pt(ts,df=srmom$n-1,lower.tail=FALSE))
 		statistic <- ts
 	} else {
-		T2 <- subprob$n * t(subprob$ESR) %*% solve(subprob$COC,subprob$ESR)
+		#T2 <- srmom$n * t(srmom$ESR) %*% solve(srmom$COC,srmom$ESR)
+		T2 <- t(srmom$ESR) %*% solve(srmom$COC,srmom$ESR)
 		names(T2) <- "T2"
 		pval <- switch(type,
-									 chisq = pchisq(T2,df=subprob$k,ncp=0,lower.tail=FALSE),
-									 F = pf((subprob$n-subprob$k) * T2/((subprob$n-1) * subprob$k),
-													df1=subprob$k,df2=subprob$n-subprob$k,lower.tail=FALSE))
+									 chisq = pchisq(T2,df=srmom$k,ncp=0,lower.tail=FALSE),
+									 F = pf((srmom$n-srmom$k) * T2/((srmom$n-1) * srmom$k),
+													df1=srmom$k,df2=srmom$n-srmom$k,lower.tail=FALSE))
 		statistic <- T2
 		if (alternative != "two.sided") {
 			warning("cannot perform directional tests on T^2")
@@ -211,16 +229,16 @@ sr_equality_test <- function(X,type=c("chisq","F","t"),
 	}
 
 	# attach names
-	names(subprob$k) <- "contrasts"
+	names(srmom$k) <- "contrasts"
 	method <- paste(c("test for equality of Sharpe ratio, via",type,"test"),collapse=" ")
-	names(subprob$SR) <- sapply(1:subprob$p,function(x) { paste(c("strat",x),collapse="_") })
+	names(srmom$SR) <- sapply(1:srmom$p,function(x) { paste(c("strat",x),collapse="_") })
 
 	cozeta <- 0
 	names(cozeta) <- "sum squared contrasts of SNR"
 
-	retval <- list(statistic = statistic, parameter = subprob$k,
-							 df1 = subprob$p, df2 = subprob$n, p.value = pval, 
-							 SR = subprob$SR, null.value = cozeta,
+	retval <- list(statistic = statistic, parameter = srmom$k,
+							 df1 = srmom$p, df2 = srmom$n, p.value = pval, 
+							 SR = srmom$SR, null.value = cozeta,
 							 alternative = alternative,
 							 method = method, data.name = dname)
 	class(retval) <- "htest"
@@ -578,7 +596,7 @@ power.sr_test <- function(n=NULL,zeta=NULL,sig.level=0.05,power=NULL,
 #' \deqn{H_0: \mu^{\top}\Sigma^{-1}\mu = \delta_0^2}{H0: mu' Sigma^-1 mu = delta_0^2}
 #'
 #' The default alternative hypothesis is the one-sided 
-#' \deqn{H_0: \mu^{\top}\Sigma^{-1}\mu > \delta_0^2}{H0: mu' Sigma^-1 mu > delta_0^2}
+#' \deqn{H_1: \mu^{\top}\Sigma^{-1}\mu > \delta_0^2}{H1: mu' Sigma^-1 mu > delta_0^2}
 #' but this can be set otherwise.
 #' 
 #' Note there is no 'drag' term here since this represents a linear offset of
@@ -590,7 +608,8 @@ power.sr_test <- function(n=NULL,zeta=NULL,sig.level=0.05,power=NULL,
 #'             zeta.s=0,ope=1,conf.level=0.95)
 #'
 #' @param X a (non-empty) numeric matrix of data values, each row independent,
-#        each column representing an asset.
+#'       each column representing an asset, or an object of 
+#'       class \code{sropt}.
 #' @param alternative a character string specifying the alternative hypothesis,
 #'       must be one of \code{"two.sided"}, \code{"greater"} (default) or
 #'       \code{"less"}.  You can specify just the initial letter.
@@ -620,6 +639,16 @@ power.sr_test <- function(n=NULL,zeta=NULL,sig.level=0.05,power=NULL,
 #' plot(ecdf(pvs))
 #' abline(0,1,col='red') 
 #' 
+#' # input a sropt objects:
+#' nfac <- 5
+#' nyr <- 10
+#' ope <- 253
+#' # simulations with no covariance structure.
+#' # under the null:
+#' set.seed(as.integer(charToRaw("be determinstic")))
+#' Returns <- matrix(rnorm(ope*nyr*nfac,mean=0,sd=0.0125),ncol=nfac)
+#' asro <- as.sropt(Returns,drag=0,ope=ope)
+#' stest <- sropt_test(asro,alternative="two.sided")
 #'
 #'@export
 sropt_test <- function(X,alternative=c("greater","two.sided","less"),
@@ -633,7 +662,13 @@ sropt_test <- function(X,alternative=c("greater","two.sided","less"),
 		stop("'conf.level' must be a single number between 0 and 1")
 
 	dname <- deparse(substitute(X))
-	subtest <- as.sropt(X,ope=ope)
+	if (is.sropt(X)) {
+		subtest <- X
+		if (!is.null(ope))  
+			subtest <- reannualize(subtest,new.ope=ope)
+	} else {
+		subtest <- as.sropt(X,ope=ope)
+	}
 	statistic <- subtest$T2
 	names(statistic) <- "T2"
 	estimate <- subtest$sropt
@@ -658,6 +693,7 @@ sropt_test <- function(X,alternative=c("greater","two.sided","less"),
 	names(df1) <- "df1"
 	names(df2) <- "df2"
 	df <- c(df1,df2)
+	names(zeta.s) <- "optimal signal-noise ratio"
 	#attr(cint, "conf.level") <- conf.level
 	retval <- list(statistic = statistic, parameter = df,
 								 estimate = estimate, p.value = pval, 
@@ -777,5 +813,52 @@ power.sropt_test <- function(df1=NULL,df2=NULL,zeta.s=NULL,
 
 #UNFOLD
 
+# spanning tests #FOLDUP
+# ' @title spanning test for sub-portfolio.
+# '
+# ' @description 
+# '
+# ' Performs a test of portfolio spanning.
+# '
+# ' @details 
+# '
+# ' Suppose \eqn{x_i}{xi} are \eqn{n}{n} independent draws of a \eqn{q}{q}-variate
+# ' normal random variable with mean \eqn{\mu}{mu} and covariance matrix
+# ' \eqn{\Sigma}{Sigma}. Let \eqn{G} be some \eqn{n_g \times p}{n_g x p} matrix
+# ' of rank \eqn{n_g}{n_g}. Let 
+# ' \deqn{\Delta\zeta_{*} = \max_{w\,: G\Sigma w = 0}\frac{w^{\top}\mu}{\sqrt{w^{\top}\Sigma w}}}{Delta zeta* = max {(w'mu)/sqrt(w'Sigma w)|G Sigma w = 0}}
+# '
+# ' A spanning test tests the hypothesis
+# ' \deqn{H_0: \Delta \zeta_{*} = 0}{H0: Delta zeta* = 0}
+# ' against the alternative hypothesis
+# ' \deqn{H_1: \Delta \zeta_{*} > 0}{H0: Delta zeta* > 0}
+# '
+# ' An alternative formulation, is as follows. Let
+# ' \deqn{\zeta_{*,I} = \max_{w\,: w = I v}\frac{w^{\top}\mu}{\sqrt{w^{\top}\Sigma w}}}{zeta*,I = max {(w'mu)/sqrt(w'Sigma w)|w = I v}}
+# ' and, similarly, let
+# ' \deqn{\zeta_{*,G} = \max_{w\,: w = G v}\frac{w^{\top}\mu}{\sqrt{w^{\top}\Sigma w}}}{zeta*,G = max {(w'mu)/sqrt(w'Sigma w)|w = G v}}
+# '
+# ' Then
+# ' \deqn{\left(\Delta\zeta_{*}\right)^2 = \zeta_{*,I}^2 - \zeta_{*,G}^2}{(Delta zeta*)^2 = zeta*,I^2 - zeta*,G^2}
+# ' And so the spanning test tests 
+# ' \deqn{H_0: \zeta_{*,I}^2 = \zeta_{*,G}^2}{H0: zeta*,I^2 = zeta*,G^2}
+# ' against the alternative hypothesis
+# ' \deqn{H_1: \zeta_{*,I}^2 > \zeta_{*,G}^2}{H0: zeta*,I^2 > zeta*,G^2}
+# '
+# ' The spanning test also provides estimates and confidence intervals on the
+# ' difference 
+# ' \eqn{\zeta_{*,I}^2 - \zeta_{*,G}^2}{zeta*,I^2 - zeta*,G^2}
+# ' 
+# ' @usage
+# '
+# ' # 2FIX: start here ... 
+# '
+# ' @template etc
+# ' @template sropt
+# ' @template ref-JW
+# ' @export
+
+#UNFOLD
+
 #for vim modeline: (do not edit)
 # vim:fdm=marker:fmr=FOLDUP,UNFOLD:cms=#%s:syn=r:ft=r
